package uy.gub.imm;

import java.util.HashMap;

import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.apache.log4j.Logger;
import org.hornetq.api.core.HornetQException;
import org.hornetq.api.core.Interceptor;
import org.hornetq.api.core.Message;
import org.hornetq.api.core.SimpleString;
import org.hornetq.api.core.management.ObjectNameBuilder;
import org.hornetq.api.jms.management.JMSConnectionInfo;
import org.hornetq.api.jms.management.JMSServerControl;
import org.hornetq.core.protocol.core.Packet;
import org.hornetq.core.protocol.core.impl.wireformat.SessionSendMessage;
import org.hornetq.spi.core.protocol.RemotingConnection;

public class SimpleInterceptor implements Interceptor {

   private static final String JMX_URL = "service:jmx:rmi://127.0.0.1/jndi/rmi://127.0.0.1:1090/jmxconnector";   
   private static Logger log = Logger.getLogger(SimpleInterceptor.class);

   @Override
	public boolean intercept(Packet packet, RemotingConnection arg1)
			throws HornetQException {

	   if (packet instanceof SessionSendMessage) {
			JMXConnector connector = null;
			JMSServerControl serverControl = null;
			
			try {
				ObjectName on = ObjectNameBuilder.DEFAULT.getJMSServerObjectName(); 
				connector = JMXConnectorFactory.connect(new JMXServiceURL(JMX_URL), new HashMap<String, Object>());
		         MBeanServerConnection mbsc = connector.getMBeanServerConnection();
		         
		         serverControl = (JMSServerControl)MBeanServerInvocationHandler.newProxyInstance(mbsc,
                         on, JMSServerControl.class, false);			
		         String jsonString = serverControl.listConnectionsAsJSON();
			
		         JMSConnectionInfo infos[] = JMSConnectionInfo.from(jsonString);
		         for (JMSConnectionInfo info : infos){
		        	 if (info.getConnectionID().equals(arg1.getTransportConnection().getID().toString())){
		     			SessionSendMessage realPacket = (SessionSendMessage) packet;
		    			Message msg = realPacket.getMessage();
		    			String user = info.getUsername();
		    			if (user != null){ 
		    				log.info("User " + user);
		    				msg.putStringProperty(new SimpleString("user_id"), new SimpleString(user));
		    			}
		        		 break;
		        	 }
		         }
			} catch (Exception e) {
				e.printStackTrace();
			} finally{
		         if (connector != null){
		        	 try{connector.close();}catch(Exception e1){}				
		         }
			}
		}
		
		return true;
	}

}
